@@ -258,6 +258,11 @@ int main(int argc, char *argv[]) {
 #else
     fprintf (out, "(sb-int:style-warn \"Couldn't grovel for EPROTONOSUPPORT (unknown to the C compiler).\")\n");
 #endif
+#ifdef ERANGE
+    fprintf (out, "(cl:defconstant ERANGE %d \"NIL\")\n", ERANGE);
+#else
+    fprintf (out, "(sb-int:style-warn \"Couldn't grovel for ERANGE (unknown to the C compiler).\")\n");
+#endif
 #ifdef ESOCKTNOSUPPORT
     fprintf (out, "(cl:defconstant ESOCKTNOSUPPORT %d \"NIL\")\n", ESOCKTNOSUPPORT);
 #else
@@ -405,9 +410,15 @@ int main(int argc, char *argv[]) {
     fprintf (out, "(cl:declaim (cl:inline GETPROTOBYNAME))\n");
     fprintf (out, "(sb-grovel::define-foreign-routine (\"getprotobyname\" GETPROTOBYNAME)\n");
     fprintf (out, "  (* PROTOENT)\n  (NAME C-STRING))\n");
+    fprintf (out, "(cl:declaim (cl:inline GETPROTOBYNAME-R))\n");
+    fprintf (out, "(sb-grovel::define-foreign-routine (\"getprotobyname_r\" GETPROTOBYNAME-R)\n");
+    fprintf (out, "  INT\n  (NAME C-STRING)\n  (RESULT_BUF (* PROTOENT))\n  (BUFFER (* CHAR))\n  (BUFFER-LEN SIZE-T)\n  (RESULT (* (* PROTOENT))))\n");
     fprintf (out, "(cl:declaim (cl:inline GETPROTOBYNUMBER))\n");
     fprintf (out, "(sb-grovel::define-foreign-routine (\"getprotobynumber\" GETPROTOBYNUMBER)\n");
     fprintf (out, "  (* PROTOENT)\n  (PROTO INT))\n");
+    fprintf (out, "(cl:declaim (cl:inline GETPROTOBYNUMBER-R))\n");
+    fprintf (out, "(sb-grovel::define-foreign-routine (\"getprotobynumber_r\" GETPROTOBYNUMBER-R)\n");
+    fprintf (out, "  INT\n  (PROTO INT)\n  (RESULT_BUF (* PROTOENT))\n  (BUFFER (* CHAR))\n  (BUFFER-LEN SIZE-T)\n  (RESULT (* (* PROTOENT))))\n");
 #ifdef INADDR_ANY
     fprintf (out, "(cl:defconstant INADDR-ANY %d \"NIL\")\n", INADDR_ANY);
 #else
