@@ -81,8 +81,7 @@
         ;; become COMPILE instead of EVAL, which seems nicer to me.
         (eval `(function ,object)))
        ((instance-lambda)
-        (deprecation-warning 'instance-lambda 'lambda)
-        (eval `(function ,object)))
+        (deprecation-error "0.9.3.32" 'instance-lambda 'lambda))
        (t
         (error 'simple-type-error
                :datum object
