@@ -33,7 +33,9 @@
                   ;; pseudo-atomicity too, but they handle it without
                   ;; messing with special variables.)
                   #!+(or x86 x86-64) *pseudo-atomic-bits*
+                  #!+(or hpux) sb!vm::*c-lra*
                   *allow-with-interrupts*
+                  sb!unix::*unblock-deferrables-on-enabling-interrupts-p*
                   *interrupts-enabled*
                   *interrupt-pending*
                   *free-interrupt-context-index*
