@@ -45,3 +45,23 @@
     (ash -1 (- n-word-bits n-lowtag-bits))
   #!+sb-doc
   "the fixnum closest in value to negative infinity")
+
+(def!constant most-positive-exactly-single-float-fixnum
+  (min #xffffff sb!xc:most-positive-fixnum))
+(def!constant most-negative-exactly-single-float-fixnum
+  (max #x-ffffff sb!xc:most-negative-fixnum))
+(def!constant most-positive-exactly-double-float-fixnum
+  (min #x1fffffffffffff sb!xc:most-positive-fixnum))
+(def!constant most-negative-exactly-double-float-fixnum
+  (max #x-1fffffffffffff sb!xc:most-negative-fixnum))
+
+;;;; Point where continuous area starting at dynamic-space-start bumps into
+;;;; next space.
+#!+gencgc
+(def!constant max-dynamic-space-end
+    (let ((stop (1- (ash 1 n-word-bits)))
+          (start dynamic-space-start))
+      (dolist (other-start (list read-only-space-start static-space-start linkage-table-space-start))
+        (when (< start other-start)
+          (setf stop (min stop other-start))))
+      stop))
