@@ -41,7 +41,7 @@
       (setf (%simple-fun-name res) (entry-info-name entry-info))
       (setf (%simple-fun-arglist res) (entry-info-arguments entry-info))
       (setf (%simple-fun-type res) (entry-info-type entry-info))
-      (setf (%simple-fun-xrefs res) (entry-info-xref entry-info))
+      (setf (%simple-fun-info res) (entry-info-info entry-info))
 
       (note-fun entry-info res object))))
 
