@@ -70,10 +70,10 @@
   (defreg lip 31) ; Lisp interior pointer
 
   (defregset non-descriptor-regs
-      nl0 nl1 nl2 nl3 nl4 cfunc nargs)
+      nl0 nl1 nl2 nl3 nl4 cfunc nargs nfp)
 
   (defregset descriptor-regs
-      a0 a1 a2 a3 a4 a5 fdefn lexenv nfp ocfp lra l0 l1)
+      a0 a1 a2 a3 a4 a5 fdefn lexenv ocfp lra l0 l1)
 
   (defregset *register-arg-offsets*
       a0 a1 a2 a3 a4 a5)
@@ -107,15 +107,14 @@
                                                          "-SC-NUMBER"))))
                 (list* `(define-storage-class ,sc-name ,index
                           ,@(cdr class))
-                       `(defconstant ,constant-name ,index)
-                       `(export ',constant-name)
+                       `(def!constant ,constant-name ,index)
                        forms)))
        (index 0 (1+ index))
        (classes classes (cdr classes)))
       ((null classes)
        (nreverse forms))))
 
-(def!constant kludge-nondeterministic-catch-block-size 7)
+(def!constant kludge-nondeterministic-catch-block-size 6)
 
 (!define-storage-classes
 
@@ -162,7 +161,8 @@
   (character-stack non-descriptor-stack) ; non-descriptor characters.
   (sap-stack non-descriptor-stack) ; System area pointers.
   (single-stack non-descriptor-stack) ; single-floats
-  (double-stack non-descriptor-stack :element-size 2) ; double floats.
+  (double-stack non-descriptor-stack
+                :element-size 2 :alignment 2) ; double floats.
   ;; complex-single-floats
   (complex-single-stack non-descriptor-stack :element-size 2)
   ;; complex-double-floats.
@@ -306,7 +306,7 @@
          (sc-number-or-lose 'immediate)
          nil))
     ((or (integer #.sb!xc:most-negative-fixnum #.sb!xc:most-positive-fixnum)
-         system-area-pointer character)
+         character)
      (sc-number-or-lose 'immediate))
     (system-area-pointer
      (sc-number-or-lose 'immediate))
@@ -318,20 +318,20 @@
 
 ;;; The SC numbers for register and stack arguments/return values.
 ;;;
-(defconstant register-arg-scn (meta-sc-number-or-lose 'descriptor-reg))
-(defconstant immediate-arg-scn (meta-sc-number-or-lose 'any-reg))
-(defconstant control-stack-arg-scn (meta-sc-number-or-lose 'control-stack))
+(def!constant register-arg-scn (meta-sc-number-or-lose 'descriptor-reg))
+(def!constant immediate-arg-scn (meta-sc-number-or-lose 'any-reg))
+(def!constant control-stack-arg-scn (meta-sc-number-or-lose 'control-stack))
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
 
 ;;; Offsets of special stack frame locations
-(defconstant ocfp-save-offset 0)
-(defconstant lra-save-offset 1)
-(defconstant nfp-save-offset 2)
+(def!constant ocfp-save-offset 0)
+(def!constant lra-save-offset 1)
+(def!constant nfp-save-offset 2)
 
 ;;; The number of arguments/return values passed in registers.
 ;;;
-(defconstant register-arg-count 6)
+(def!constant register-arg-count 6)
 
 ;;; The offsets within the register-arg SC that we pass values in, first
 ;;; value first.
@@ -354,7 +354,7 @@
           *register-arg-offsets*))
 
 ;;; This is used by the debugger.
-(defconstant single-value-return-byte-offset 8)
+(def!constant single-value-return-byte-offset 8)
 
 ;;; This function is called by debug output routines that want a pretty name
 ;;; for a TN's location.  It returns a thing that can be printed with PRINC.
