@@ -21,6 +21,24 @@
   (:generator 5
     (inst b dest)
     (inst nop)))
+
+
+;;;; Generic conditional VOPs
+
+;;; The generic conditional branch, emitted immediately after test
+;;; VOPs that only set flags.
+
+(define-vop (branch-if)
+  (:info dest flags not-p)
+  (:ignore dest flags not-p)
+  (:generator 0
+     (error "BRANCH-IF not yet implemented")))
+
+(!def-vm-support-routine
+    convert-conditional-move-p (node dst-tn x-tn y-tn)
+  (declare (ignore node dst-tn x-tn y-tn))
+  nil)
+
 
 ;;;; conditional VOPs:
 
