@@ -19,9 +19,9 @@
 #include "sbcl.h"
 
 #ifdef LISP_FEATURE_GENCGC
-#define PAGE_BYTES GENCGC_PAGE_SIZE
+#define PAGE_BYTES GENCGC_PAGE_BYTES
 #else
-#define PAGE_BYTES 0x1000
+#define PAGE_BYTES BACKEND_PAGE_BYTES
 #endif
 
 typedef signed long page_index_t;
@@ -40,8 +40,6 @@ extern void clear_auto_gc_trigger(void);
 
 #include "fixnump.h"
 
-#include "pseudo-atomic.h"
-
 extern boolean maybe_gc(os_context_t *context);
 
 extern unsigned long bytes_consed_between_gcs;
