@@ -18,10 +18,11 @@
  * here? (The answer wasn't obvious to me when merging the
  * architecture-abstracting patches for CSR's SPARC port. -- WHN 2002-02-15) */
 
-#include "interr.h"
+#define COMPILER_BARRIER \
+    do { __asm__ __volatile__ ( "" : : : "memory"); } while (0)
 
 static inline void
-get_spinlock(volatile lispobj *word,long value)
+get_spinlock(volatile lispobj *word, unsigned long value)
 {
 #ifdef LISP_FEATURE_SB_THREAD
     u32 eax=0;
@@ -29,13 +30,13 @@ get_spinlock(volatile lispobj *word,long value)
         lose("recursive get_spinlock: 0x%x,%ld\n",word,value);
     do {
 #if defined(LISP_FEATURE_DARWIN)
-        asm ("xor %0,%0;\n\
+        asm volatile ("xor %0,%0;\n\
               lock/cmpxchg %1,%2"
              : "=a" (eax)
              : "r" (value), "m" (*word)
              : "memory", "cc");
 #else
-        asm ("xor %0,%0\n\
+        asm volatile ("xor %0,%0\n\
               lock cmpxchg %1,%2"
              : "=a" (eax)
              : "r" (value), "m" (*word)
@@ -51,22 +52,23 @@ get_spinlock(volatile lispobj *word,long value)
 static inline void
 release_spinlock(volatile lispobj *word)
 {
+    /* See comment in RELEASE-SPINLOCK in target-thread.lisp. */
+    COMPILER_BARRIER;
     *word=0;
+    COMPILER_BARRIER;
 }
 
-#include <stdio.h>
-
 static inline lispobj
 swap_lispobjs(volatile lispobj *dest, lispobj value)
 {
     lispobj old_value;
 #if defined(LISP_FEATURE_DARWIN)
-    asm ("lock/xchg %0,(%1)"
+    asm volatile ("lock/xchg %0,(%1)"
          : "=r" (old_value)
          : "r" (dest), "0" (value)
          : "memory");
 #else
-    asm ("lock xchg %0,(%1)"
+    asm volatile ("lock xchg %0,(%1)"
          : "=r" (old_value)
          : "r" (dest), "0" (value)
          : "memory");
